#ifndef NODE_HPP
#define NODE_HPP

#include <vector>

class Node {
    using List = std::vector<unsigned>;
public:
    // Default constructor creates a leaf node
    Node() = default;
    // Whoever creates the node is responsible for deciding which particles are in which octant and for allocating the localParticles array 
    // Node will write to localParticles array
    Node(Node* root, Node* parent, double* masses, double* particles, unsigned nParticles, unsigned depth, 
        double size, double center[3], List& localParticles, unsigned nLocalParticles);
    // Destuctor
    ~Node();
    // Copy constructor (default)
    Node(const Node&) = default;
    // Copy assignment
    Node& operator=(const Node&);
    // Update tree: visit each node, check if it needs to be split or merged (for example if a particle has left the region)
    void update(List& allParticles);
    // Calculate force on a particle recursively
    double* calculateForce(unsigned particle);
    // Calculate center of mass
    void calculateCenterOfMass();
    // Function that takes a particle and returns the octant it is in. This function doesn't check if the particle is actually in the node.
    unsigned getOctant(unsigned particle);

private:
    Node* root_;
    Node* parent_;
    Node* children_[8];

    double* masses_;
    double* particles_;
    unsigned nParticles_;

    unsigned depth_;
    double size_;
    double center_[3];
    double centerOfMass_[3];

    // Contains the indices of the particles that are in this node for the particles_ array
    List localParticles_;
    unsigned nLocalParticles_;

    double mass_;
    const double theta0_ = 0.4;
    bool isLeaf_;
};

#endif /* NODE_HPP */